//  Wii Remote IR sensor test  for ATMEGA168


#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

// dxh library from https://github.com/alx741/avr_i2c
#include <i2cmaster.h>


// i2c address
#define		I2C_ADR		0xB0		// ( 0x58 << 1 )

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
//#define bit_delay_time 8.5 // bit delay for 115200 with overhead
#define bit_delay_time 102 // 9600 baud

#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay


#define i2c_delay() _delay_ms(50)

#define serial_port PORTD
#define serial_direction DDRD
#define serial_pins PIND
// TX/RX from computer's perspective. in = computer sends TX. out = computer receives on RX.
#define serial_pin_in (1 << PD1)
#define serial_pin_out (1 << PD0)

#define max_buffer 25


#define led_port PORTC
#define led_direction DDRC
#define led_pin_1 (1 << PC1)
#define led_pin_2 (1 << PC0)
#define led_delay() _delay_ms(100) // LED delay


#define SCL_CLOCK  100000L // I2C clock in Hz
#define IR_CAMERA_ADDRESS 0xB0 // 8 bit address (0x58 << 1)



//
// SERIAL
//

  
void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

void put_string(volatile unsigned char *port, unsigned char pin, char *str) {
   //
   // print a null-terminated string
   //
   static int index;
   index = 0;
   do {
      put_char(port, pin, str[index]);
      ++index;
      } while (str[index] != 0);
   }






int main()
{
	int i;
	int r = 0;
	unsigned char data_buf[16];
	int p0,p1,p2,p3;
	int Ix1,Iy1,Ix2,Iy2;
	int Ix3,Iy3,Ix4,Iy4;
	int s;
	char buf[255];

	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	
	//
	// initialize output pins
	//
	set(serial_port, serial_pin_out);
	output(serial_direction, serial_pin_out);
	
	output(led_port, led_pin_1);
	output(led_port, led_pin_2);
	clear(led_port, led_pin_1);
	clear(led_port, led_pin_2);
	
	
	i2c_init(); // i2c_master_init();

	// IR sensor initialize
	
	//p0=0x72; p1=0x20; p2=0x1F; p3=0x03; // sensitivity
	//p0=0xC8; p1=0x36; p2=0x35; p3=0x03;
	  p0=0xAA; p1=0x64; p2=0x63; p3=0x03;
	//p0=0x96; p1=0xB4; p2=0xB3; p3=0x04;
	//p0=0x96; p1=0xFE; p2=0xFE; p3=0x05;

	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x30 ); i2c_write( 0x01 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();
	
	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x00 );
	i2c_write( 0x02 ); i2c_write( 0x00 ); i2c_write( 0x00 ); i2c_write( 0x71 );
	i2c_write( 0x01 ); i2c_write( 0x00 ); i2c_write( p0 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();

	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x07 ); i2c_write( 0x00 ); i2c_write( p1 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();

	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x1A ); i2c_write( p2 ); i2c_write( p3 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();

	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x33 ); i2c_write( 0x03 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();

	i2c_start_wait(I2C_ADR+I2C_WRITE);
	i2c_write( 0x30 ); i2c_write( 0x08 );
	i2c_stop();
	if (r!=0) { rs_puts("Error: no ACK\n"); }
	i2c_delay();


	while (1) {
				
		i2c_start_wait(I2C_ADR+I2C_WRITE);
		i2c_write( 0x36 );
		i2c_stop();
		if (r!=0) { rs_puts("Error: no ACK\n"); }
		
		i2c_start_wait(I2C_ADR+I2C_READ);
		for (i=0;i<16;i++) { data_buf[i]=0; }
		for (i=0;i<16;i++) {
			if (i!=15) {
				data_buf[i] = i2c_readAck();
			} else {
				data_buf[i] = i2c_readNak();
			}
		}
		i2c_stop();
		if (r!=0) { rs_puts("Error: no ACK (2)\n"); }
		
		
		Ix1 = data_buf[1]; Iy1 = data_buf[2]; s = data_buf[3];
		Ix1 += (s & 0x30) <<4; Iy1 += (s & 0xC0) <<2;
		Ix2 = data_buf[4]; Iy2 = data_buf[5]; s = data_buf[6];
		Ix2 += (s & 0x30) <<4; Iy2 += (s & 0xC0) <<2;
		Ix3 = data_buf[7]; Iy3 = data_buf[8]; s = data_buf[9];
		Ix3 += (s & 0x30) <<4; Iy3 += (s & 0xC0) <<2;
		Ix4 = data_buf[10]; Iy4 = data_buf[11]; s = data_buf[12];
		Ix4 += (s & 0x30) <<4; Iy4 += (s & 0xC0) <<2;


		/* put_char(&serial_port, serial_pin_out, 1); */
		/* put_char(&serial_port, serial_pin_out, 2); */
		/* put_char(&serial_port, serial_pin_out, 3); */
		/* put_char(&serial_port, serial_pin_out, 4); */

		
		put_char(&serial_port, serial_pin_out, data_buf[0]);
		put_char(&serial_port, serial_pin_out, data_buf[1]);
		put_char(&serial_port, serial_pin_out, data_buf[2]);
		put_char(&serial_port, serial_pin_out, data_buf[3]);
		put_char(&serial_port, serial_pin_out, data_buf[4]);
		put_char(&serial_port, serial_pin_out, data_buf[5]);
		put_char(&serial_port, serial_pin_out, data_buf[6]);
		put_char(&serial_port, serial_pin_out, data_buf[7]);
		put_char(&serial_port, serial_pin_out, data_buf[8]);
		put_char(&serial_port, serial_pin_out, data_buf[9]);
		put_char(&serial_port, serial_pin_out, data_buf[10]);
		put_char(&serial_port, serial_pin_out, data_buf[11]);
		put_char(&serial_port, serial_pin_out, data_buf[12]);
		put_char(&serial_port, serial_pin_out, data_buf[13]);
		put_char(&serial_port, serial_pin_out, data_buf[14]);
		put_char(&serial_port, serial_pin_out, data_buf[15]);

		
		/* put_char(&serial_port, serial_pin_out, 5); */
		/* put_char(&serial_port, serial_pin_out, 6); */
		/* put_char(&serial_port, serial_pin_out, 7); */
		/* put_char(&serial_port, serial_pin_out, 8); */
		
		/* sprintf(buf,"%d," ,Ix1); rs_puts(buf); */
		/* sprintf(buf,"%d," ,Iy1); rs_puts(buf); */
		/* sprintf(buf,"%d," ,Ix2); rs_puts(buf); */
		/* sprintf(buf,"%d\n",Iy2); rs_puts(buf); */

		i2c_delay();
		i2c_delay();

	}
	
	return 0;
}
